export class WebGoogleMaps {
    constructor() {
        this.googleMapsLoaded = false;
    }
    async componentDidLoad() {
        await this.loadGoogleMapsApi();
    }
    async loadGoogleMapsApi() {
        if (this.googleMapsLoaded || !this.apiKey) {
            return;
        }
        const scripts = document.getElementById('webGoogleMapsApiKey');
        if (scripts) {
            await this.initGoogleMaps();
            return;
        }
        const script = document.createElement('script');
        script.onload = () => {
            this.googleMapsApiKeyLoaded.emit();
        };
        script.id = 'webGoogleMapsApiKey';
        script.src = 'https://maps.googleapis.com/maps/api/js?key=' + this.apiKey;
        script.defer = true;
        document.head.appendChild(script);
    }
    async initGoogleMaps() {
        return new Promise(async (resolve) => {
            if (typeof google !== 'object' || !google || !google.maps) {
                resolve();
                return;
            }
            if (!this.lat || !this.lng) {
                resolve();
                return;
            }
            if (this.googleMapsLoaded) {
                resolve();
                return;
            }
            this.googleMapsLoaded = true;
            let mapOptions = this.normalizeGoogleMapsOptions();
            mapOptions.center = new google.maps.LatLng(this.lat, this.lng);
            const mapDiv = this.el.shadowRoot.querySelector('div.gmap');
            if (this.mapStyle && this.mapStyle.styles) {
                mapOptions.mapTypeControlOptions = { mapTypeIds: [google.maps.MapTypeId.ROADMAP, 'map_style'] };
                mapOptions.mapTypeControl = false;
            }
            else {
                mapOptions.mapTypeId = google.maps.MapTypeId.ROADMAP;
            }
            const map = new google.maps.Map(mapDiv, mapOptions);
            if (this.mapStyle && this.mapStyle.styles) {
                const styledMap = new google.maps.StyledMapType(this.mapStyle.styles, { name: this.mapStyle.name });
                map.mapTypes.set('map_style', styledMap);
                map.setMapTypeId('map_style');
            }
            google.maps.event.addListenerOnce(map, 'idle', async () => {
                await this.addCircles(map);
                await this.addMarkers(map);
                this.fitMapToBounds(map);
            });
            resolve();
        });
    }
    normalizeGoogleMapsOptions() {
        const mapOptions = {
            zoom: 15,
            scrollwheel: false,
            draggable: false,
            disableDoubleClickZoom: true,
            streetViewControl: false,
            clickableIcons: true,
            fullscreenControl: false
        };
        return Object.assign({}, mapOptions, this.options);
    }
    addCircles(map) {
        return new Promise((resolve) => {
            if (this.circles && this.circles.length > 0) {
                this.circles.forEach((circle) => {
                    if (!circle.center && circle.lat && circle.lng) {
                        circle.center = new google.maps.LatLng(circle.lat, circle.lng);
                    }
                    if (circle.center && circle.radius) {
                        circle.map = map;
                        new google.maps.Circle(circle);
                    }
                });
            }
            resolve();
        });
    }
    addMarkers(map) {
        return new Promise((resolve) => {
            if (this.markers && this.markers.markers && this.markers.markers.length > 0) {
                this.markers.markers.forEach((markerOptions) => {
                    if (!markerOptions.position && markerOptions.lat && markerOptions.lng) {
                        markerOptions.position = new google.maps.LatLng(markerOptions.lat, markerOptions.lng);
                    }
                    if (markerOptions.position) {
                        markerOptions.map = map;
                        const marker = new google.maps.Marker(markerOptions);
                        this.addMarkerToBounds(markerOptions.position);
                        this.addInfoWindow(map, marker, markerOptions.infoWindowOptions);
                    }
                });
            }
            this.fitMapToBounds(map);
            resolve();
        });
    }
    addMarkerToBounds(position) {
        if (!this.bounds) {
            this.bounds = new google.maps.LatLngBounds();
        }
        this.bounds.extend(position);
    }
    addInfoWindow(map, marker, infoWindowOptions) {
        if (!infoWindowOptions || !infoWindowOptions.content) {
            return;
        }
        google.maps.event.addListener(marker, 'click', () => {
            if (this.singleInfoWindow !== void 0) {
                this.singleInfoWindow.close();
            }
            this.singleInfoWindow = new google.maps.InfoWindow(infoWindowOptions);
            this.singleInfoWindow.open(map, marker);
        });
    }
    fitMapToBounds(map) {
        if (this.markers && this.markers.fitBounds && this.bounds) {
            map.fitBounds(this.bounds);
        }
    }
    render() {
        return h("div", { class: "gmap" });
    }
    static get is() { return "web-google-maps"; }
    static get encapsulation() { return "shadow"; }
    static get properties() { return {
        "apiKey": {
            "type": String,
            "attr": "api-key",
            "watchCallbacks": ["loadGoogleMapsApi"]
        },
        "circles": {
            "type": "Any",
            "attr": "circles"
        },
        "el": {
            "elementRef": true
        },
        "lat": {
            "type": Number,
            "attr": "lat",
            "watchCallbacks": ["initGoogleMaps"]
        },
        "lng": {
            "type": Number,
            "attr": "lng",
            "watchCallbacks": ["initGoogleMaps"]
        },
        "mapStyle": {
            "type": "Any",
            "attr": "map-style"
        },
        "markers": {
            "type": "Any",
            "attr": "markers"
        },
        "options": {
            "type": "Any",
            "attr": "options"
        }
    }; }
    static get events() { return [{
            "name": "googleMapsApiKeyLoaded",
            "method": "googleMapsApiKeyLoaded",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get listeners() { return [{
            "name": "document:googleMapsApiKeyLoaded",
            "method": "initGoogleMaps"
        }]; }
    static get style() { return "/**style-placeholder:web-google-maps:**/"; }
}
